@@ -24,6 +24,7 @@ HERTZ = "Hz"
 VOLTAMPEREREACTIVE = "VAr"
 VOLTAMPEREREACTIVE_HOUR = "VArh"
 VOLTAMPERE = "VA"
+PER_KILOWATTHOUR = "{}/kWh"
 
 
 class API_VERSION(enum.Enum):
@@ -75,6 +76,13 @@ URL_DEVICE_INVERTER_COMMON = {
         "DataCollection=CommonInverterData"
     ),
 }
+URL_ACTIVE_DEVICE_INFO_SYSTEM = {
+    API_VERSION.V1: "GetActiveDeviceInfo.cgi?DeviceClass=System"
+}
+URL_LOGGER_INFO = {
+    API_VERSION.V0: "GetLoggerInfo.cgi",
+    API_VERSION.V1: "GetLoggerInfo.cgi",
+}
 
 
 class NotSupportedError(ValueError):
@@ -102,6 +110,9 @@ class Fronius:
         """
         self._aio_session = session
         self.url = re.sub(r"/+$", "", url)
+        # prepend http:// if missing, by fronius API this is the only supported protocol
+        if not self.url.startswith("http"):
+            self.url = "http://{}".format(self.url)
         self.api_version = api_version
         self.base_url = API_BASEPATHS.get(API_VERSION)
 
@@ -180,6 +191,8 @@ class Fronius:
 
     async def fetch(
         self,
+        active_device_info=True,
+        logger_info=True,
         power_flow=True,
         system_meter=True,
         system_inverter=True,
@@ -190,6 +203,10 @@ class Fronius:
         loop=None,
     ):
         requests = []
+        if active_device_info:
+            requests.append(self.current_active_device_info())
+        if logger_info:
+            requests.append(self.current_logger_info())
         if power_flow:
             requests.append(self.current_power_flow())
         if system_meter:
@@ -244,11 +261,23 @@ class Fronius:
 
         try:
             sensor.update(Fronius._status_data(res))
-            # TODO use update here as well
-            sensor = fun(sensor, res["Body"]["Data"])
         except (TypeError, KeyError):
             # break if Data is empty
-            _LOGGER.info("No data returned from {}".format(spec))
+            _LOGGER.info(
+                "No header data returned from {} ({})".format(spec, spec_formattings)
+            )
+        try:
+            # TODO use update here as well
+            fun(sensor, res["Body"]["Data"])
+        except (TypeError, KeyError):
+            # LoggerInfo oddly deviates from the default scheme
+            try:
+                fun(sensor, res["Body"]["LoggerInfo"])
+            except (TypeError, KeyError):
+                # break if Data is empty
+                _LOGGER.info(
+                    "No body data returned from {} ({})".format(spec, spec_formattings)
+                )
         return sensor
 
     async def current_power_flow(self):
@@ -314,6 +343,24 @@ class Fronius:
             Fronius._system_led_data, URL_SYSTEM_LED, "current led"
         )
 
+    async def current_active_device_info(self):
+        """
+        Get info about the current active devices in a smart meter system.
+        """
+        return await self._current_data(
+            Fronius._system_active_device_info,
+            URL_ACTIVE_DEVICE_INFO_SYSTEM,
+            "current active device info",
+        )
+
+    async def current_logger_info(self):
+        """
+        Get the current logger info of a smart meter system.
+        """
+        return await self._current_data(
+            Fronius._logger_info, URL_LOGGER_INFO, "current logger info"
+        )
+
     @staticmethod
     def _system_led_data(sensor, data):
         _LOGGER.debug("Converting system led data: '{}'".format(data))
@@ -531,7 +578,18 @@ class Fronius:
                 "unit": data["UDC_2"]["Unit"],
             }
         if "DeviceStatus" in data:
-            sensor["inverter_state"] = {"value": data["DeviceStatus"]["InverterState"]}
+            if "InverterState" in data["DeviceStatus"]:
+                sensor["inverter_state"] = {
+                    "value": data["DeviceStatus"]["InverterState"]
+                }
+            if "ErrorCode" in data["DeviceStatus"]:
+                sensor["error_code"] = {"value": data["DeviceStatus"]["ErrorCode"]}
+            if "StatusCode" in data["DeviceStatus"]:
+                sensor["status_code"] = {"value": data["DeviceStatus"]["StatusCode"]}
+            if "LEDState" in data["DeviceStatus"]:
+                sensor["led_state"] = {"value": data["DeviceStatus"]["LEDState"]}
+            if "LEDColor" in data["DeviceStatus"]:
+                sensor["led_color"] = {"value": data["DeviceStatus"]["LEDColor"]}
 
         return sensor
 
@@ -853,3 +911,116 @@ class Fronius:
             module["serial"] = {"value": data["Details"]["Serial"]}
 
         return module
+
+    @staticmethod
+    def _system_active_device_info(sensor: dict, data: dict):
+        _LOGGER.debug("Converting system active device data: '{}'".format(data))
+
+        if "Inverter" in data:
+            inverters = []
+            for device_id, device in data["Inverter"].items():
+                inverter = {"device_id": device_id, "device_type": device["DT"]}
+                if "Serial" in device:
+                    inverter["serial_number"] = device["Serial"]
+                inverters.append(inverter)
+            sensor["inverters"] = inverters
+
+        if "Meter" in data:
+            meters = []
+            for device_id, device in data["Meter"].items():
+                meter = {"device_id": device_id}
+                if "Serial" in device:
+                    meter["serial_number"] = device["Serial"]
+                meters.append(meter)
+            sensor["meters"] = meters
+
+        if "Ohmpilot" in data:
+            ohmpilots = []
+            for device_id, device in data["Ohmpilot"].items():
+                ohmpilot = {"device_id": device_id}
+                if "Serial" in device:
+                    ohmpilot["serial_number"] = device["Serial"]
+                ohmpilots.append(ohmpilot)
+            sensor["ohmpilots"] = ohmpilots
+
+        if "SensorCard" in data:
+            sensor_cards = []
+            for device_id, device in data["SensorCard"].items():
+                sensor_card = {"device_id": device_id, "device_type": device["DT"]}
+                if "Serial" in device:
+                    sensor_card["serial_number"] = device["Serial"]
+                sensor_card["channel_names"] = list(
+                    map(lambda x: x.lower().replace(" ", "_"), device["ChannelNames"])
+                )
+                sensor_cards.append(sensor_card)
+            sensor["sensor_cards"] = sensor_cards
+
+        if "Storage" in data:
+            storages = []
+            for device_id, device in data["Storage"].items():
+                storage = {"device_id": device_id}
+                if "Serial" in device:
+                    storage["serial_number"] = device["Serial"]
+                storages.append(storage)
+            sensor["storages"] = storages
+
+        if "StringControl" in data:
+            string_controls = []
+            for device_id, device in data["StringControl"].items():
+                string_control = {"device_id": device_id}
+                if "Serial" in device:
+                    string_control["serial_number"] = device["Serial"]
+                string_controls.append(string_control)
+            sensor["string_controls"] = string_controls
+
+        return sensor
+
+    @staticmethod
+    def _logger_info(sensor, data):
+        _LOGGER.debug("Converting Logger info: '{}'".format(data))
+
+        if "CO2Factor" in data and "CO2Unit" in data:
+            sensor["co2_factor"] = {
+                "value": data["CO2Factor"],
+                "unit": PER_KILOWATTHOUR.format(data["CO2Unit"]),
+            }
+
+        if "CashFactor" in data and "CashCurrency" in data:
+            # which unit does this have?
+            sensor["cash_factor"] = {
+                "value": data["CashFactor"],
+                "unit": PER_KILOWATTHOUR.format(data["CashCurrency"]),
+            }
+
+        if "DeliveryFactor" in data and "CashCurrency" in data:
+            # which unit does this have?
+            sensor["delivery_factor"] = {
+                "value": data["DeliveryFactor"],
+                "unit": PER_KILOWATTHOUR.format(data["CashCurrency"]),
+            }
+
+        if "HWVersion" in data:
+            sensor["hardware_version"] = {"value": data["HWVersion"]}
+
+        if "SWVersion" in data:
+            sensor["software_version"] = {"value": data["SWVersion"]}
+
+        if "PlatformID" in data:
+            sensor["hardware_platform"] = {"value": data["PlatformID"]}
+
+        if "ProductID" in data:
+            sensor["product_type"] = {"value": data["ProductID"]}
+
+        if "TimezoneLocation" in data:
+            sensor["time_zone_location"] = {"value": data["TimezoneLocation"]}
+
+        if "TimezoneName" in data:
+            sensor["time_zone"] = {"value": data["TimezoneName"]}
+
+        if "UTCOffset" in data:
+            sensor["utc_offset"] = {"value": data["UTCOffset"]}
+
+        if "UniqueID" in data:
+            sensor["unique_identifier"] = {"value": data["UniqueID"]}
+
+        return sensor
